import axios from "axios";
import { LoginDetail } from "../models/loginDetails.js";
import { APIError } from "rest-api-errors";

export const login = async (username, password) => {
  try {
    const response = await axios.get(
      `${process.env.SWAGGER_URL}/master/Login?username=${username}&password=${password}`
    );
    return response.data;
  } catch (error) {
    throw new APIError(
      error?.response?.status,
      `${error?.response?.status}`,
      error?.response?.statusText
    );
  }
};

export const agencyRates = async (authorizationToken, username) => {
  try {
    const response = await axios.get(
      `${process.env.SWAGGER_URL}/master/agency-rates`,
      {
        headers: {
          Authorization: authorizationToken,
          Accept: "application/json",
          username: username,
        },
      }
    );
    return response.data;
  } catch (error) {
    throw new APIError(
      error?.response?.status,
      `${error?.response?.status}`,
      error?.response?.statusText
    );
  }
};

export const addLoginDetails = async (login_response) => {
  try {
    const loginData = {
      macAddress: login_response.Mac_Address,
      gUserId: login_response.G_UserID,
      gUserName: login_response.G_UserName,
      gUserGroup: login_response.G_UserGroup,
      gDtLogin: login_response.G_DtLogin,
      gTmLogin: login_response.G_TmLogin,
      gUserEmailId: login_response.G_UserEmailID,
      groupName: login_response.Group_Name,
      passExpiryDt: login_response.PassExpiryDt,
      gBranchName: login_response.G_Branch_Name,
      gBranchId: login_response.G_Branch_ID,
      baseCurrencyId: login_response.Base_Currency_ID,
      currencyShortName: login_response.CurrencyShortName,
      baseCountryId: login_response.Base_Country_ID,
      isArabic: login_response.IsArebic,
      isSubAgent: login_response.IsSubAgent,
      isSendSpecific: login_response.IsSendSpecific,
      isRecSpecific: login_response.IsRecSpecific,
      isManualEntry: login_response.IsManualEntry,
      subUserGroupId: login_response.SubUserGroup_Id,
      subGroupName: login_response.SubGroup_Name,
      gAgentId: login_response.G_Agent_ID,
      countryName: login_response.CountryName,
      agentCountryId: login_response.Agent_Country_ID,
      agentCityId: login_response.Agent_City_ID,
      agentLocationId: login_response.Agent_Location_ID,
      agentCurrencyId: login_response.Agent_Currency_ID,
      sendingSettlementCurrency: login_response.Sending_SettlementCurrency,
      sendingSttlementCcyName: login_response.Sending_SttlementCCYName,
      individualTransactionLimit: login_response.IndividualTransactionLimit,
      agentCountryTimeDiff: login_response.AgentCountryTimeDiff,
      agentBranchId: login_response.Agent_Branch_ID,
      transactionYearStartDate: login_response.TransactionYearStartDate,
      transactionYearEndDate: login_response.TransactionYearEndDate,
      isHeadOffice: login_response.IsHeadOffice,
      treeMainNode: login_response.TreeMainNode,
      gBranchAddress: login_response.G_Branch_Address,
      gPhone: login_response.G_Phone,
      gFax: login_response.G_Fax,
      gEmail: login_response.G_Email,
      countComplaint: login_response.CountComplaint,
      gInformation: login_response.G_Information,
      currentLoginId: login_response.CurrentLogin_ID,
    };
    let loginDetail = new LoginDetail(loginData);
    return await loginDetail.save();
  } catch (error) {}
};

export const getAllCurrencies = async (
  authorizationToken,
  username,
  currencyId = ""
) => {
  try {
    const requestUrl = `${process.env.SWAGGER_URL}master/currency${
      currencyId ? `?currencyId=${currencyId}` : ""
    }`;
    const response = await axios.get(requestUrl, {
      headers: {
        Authorization: authorizationToken,
        Accept: "application/json",
        username: username,
      },
    });
    return response.data;
  } catch (error) {
    throw new APIError(
      error?.response?.status,
      `${error?.response?.status}`,
      error?.response?.statusText
    );
  }
};

export const addCurrencies = async (authorizationToken, username, payload) => {
  try {
    const response = await axios.post(
      `${process.env.SWAGGER_URL}/master/currency`,
      payload,
      {
        headers: {
          Authorization: authorizationToken,
          Accept: "application/json",
          "Content-Type": "application/json",
          username: username,
        },
      }
    );
    return response.data;
  } catch (error) {
    throw new APIError(
      error?.response?.status,
      `${error?.response?.status}`,
      error?.response?.statusText
    );
  }
};

export const updateCurrencies = async (
  authorizationToken,
  username,
  payload
) => {
  try {
    const response = await axios.patch(
      `${process.env.SWAGGER_URL}/master/currency`,
      payload,
      {
        headers: {
          Authorization: authorizationToken,
          Accept: "application/json",
          "Content-Type": "application/json",
          username: username,
        },
      }
    );
    return response.data;
  } catch (error) {
    throw new APIError(
      error?.response?.status,
      `${error?.response?.status}`,
      error?.response?.statusText
    );
  }
};
