import { Component, OnInit, ElementRef, ViewChild } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { AllSendCorridorWiseService } from '../services/all-send-corridor-wise.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-all-sent-corridor-wise',
  templateUrl: './all-sent-corridor-wise.component.html',
  styleUrls: ['./all-sent-corridor-wise.component.css'],
})
export class AllSentCorridorWiseComponent implements OnInit {
  @ViewChild('transactionTable') transactionTable: ElementRef;
  form: FormGroup;
  countries: any[] = [];
  agents: any[] = [];
  branches: any[] = [];
  receiveCountries: any[] = [];
  receiveAgents: any[] = [];
  selectedCountry: any;
  selectedAgent: any;
  selectedBranch: any;
  selectedReceiveCountry: any;
  selectedReceiveAgent: any;
  selectedFileFormat: any = '1';
  fromDate: string;
  toDate: string;
  
  country: string;
  agent: string;
  receiveCountry: string;
  receiveAgent: string;
  isLoading: boolean = false;

  constructor(
    private fb: FormBuilder,
    private corridorWiseService: AllSendCorridorWiseService
  ) {}
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
  ngOnInit(): void {
    const currentStartDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    const currentEndDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    this.form = this.fb.group({
      fromDate: [currentStartDate, Validators.required],
      toDate: [currentEndDate, Validators.required],
      country: ['', Validators.required],
      agent: ['', Validators.required],
      branch: ['', Validators.required],
      userList: ['', Validators.required],
      receiveCountry: ['', Validators.required],
      receiveAgent: ['', Validators.required],
    });

    this.fetchCountries();
    this.fetchReceiveCountries(); // Add this line to fetch receive countries
  }
  displayByDate(): void {
    this.fromDate = this.form.get('fromDate').value;
    this.toDate = this.form.get('toDate').value;
    this.country = this.form.get('country').value;
    this.agent = this.form.get('agent').value;
    this.receiveCountry = this.form.get('receiveCountry').value;
    this.receiveAgent = this.form.get('receiveAgent').value;

    const payload = {
      SenCountryID: this.country,
      SenAgentID: this.agent,
      RecCountryID: this.receiveCountry,
      RecAgentID: this.receiveAgent,
      FromDate: this.fromDate,
      ToDate: this.toDate,
    };

    this.isLoading = true; // Start loading
    this.corridorWiseService.getAllSendReceiveTxnsGridData(payload).subscribe(
      (data: any[]) => {
        this.isLoading = false; // Stop loading
        // Handle the response data as needed
      },
      (error) => {
        this.isLoading = false; // Stop loading
        console.error('Error fetching send receive transactions:', error);
      }
    );
  }

  fetchCountries(): void {
    this.corridorWiseService.getSendCountryData(0).subscribe(
      (data: any[]) => {
        this.countries = data;
        if (this.countries.length > 0) {
          this.form.get('country').setValue(this.countries[0].CountryID);
          this.onCountryChange();
        }
      },
      (error) => {
        console.error('Error fetching countries:', error);
      }
    );
  }

  fetchReceiveCountries(): void {
    this.corridorWiseService.getReceiveCountryData().subscribe(
      (data: any[]) => {
        this.receiveCountries = data;
        if (this.receiveCountries.length > 0) {
          this.form
            .get('receiveCountry')
            .setValue(this.receiveCountries[0].CountryID);
          this.onReceiveCountryChange();
        }
      },
      (error) => {
        console.error('Error fetching receive countries:', error);
      }
    );
  }

  onCountryChange(): void {
    const selectedCountry = this.form.get('country').value;
    this.corridorWiseService.getSendAgentData(selectedCountry).subscribe(
      (data: any[]) => {
        this.agents = data;
        if (this.agents.length > 0) {
          this.form.get('agent').setValue(this.agents[0].Agent_ID);
          this.onAgentChange();
        }
      },
      (error) => {
        console.error('Error fetching agents:', error);
      }
    );
  }

  onAgentChange(): void {
    const selectedAgent = this.form.get('agent').value;

    this.corridorWiseService.getBranchData(selectedAgent, 0).subscribe(
      (data: any[]) => {
        this.branches = data;

        if (this.branches.length > 0) {
          this.form.get('branch').setValue(this.branches[0].Branch_ID);
          this.onBranchChange();
        }
      },
      (error) => {
        console.error('Error fetching branches:', error);
      }
    );
  }

  onBranchChange(): void {}

  onReceiveCountryChange(): void {
    const selectedReceiveCountry = this.form.get('receiveCountry').value;
    this.corridorWiseService
      .getReceiveAgentData(selectedReceiveCountry)
      .subscribe(
        (data: any[]) => {
          this.receiveAgents = data;
          if (this.receiveAgents.length > 0) {
            this.form
              .get('receiveAgent')
              .setValue(this.receiveAgents[0].Agent_ID);
          }
        },
        (error) => {
          console.error('Error fetching receive agents:', error);
        }
      );
  }

  onReceiveAgentChange(): void {
    const selectedReceiveCountry = this.form.get('receiveCountry').value; // Get the selected receive country from the form
    if (selectedReceiveCountry) {
      // Check if a country is selected
      this.corridorWiseService
        .getReceiveAgentData(selectedReceiveCountry)
        .subscribe(
          (data: any[]) => {
            this.receiveAgents = data;
          },
          (error) => {
            console.error('Error fetching receive agents:', error);
          }
        );
    }
  }



  onSubmit(): void {
    if (this.form.valid) {
      // Handle form submission
    } else {
      this.form.markAllAsTouched();
    }
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(
      this.transactionTable.nativeElement
    );

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Agentwise_Transaction_Sent${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }
}
