import { Component } from '@angular/core';
import { BeneficiaryComplianceHoldService } from '../services/beneficiary-compliance-hold.service';
import { Router } from '@angular/router';

@Component({
  selector: 'app-beneficiary-compliance-hold',
  templateUrl: './beneficiary-compliance-hold.component.html',
  styleUrls: ['./beneficiary-compliance-hold.component.css'],
})
export class BeneficiaryComplianceHoldComponent {
  beneficiaryNo: string = '';
  beneficiaryDetails: any[] = [];
  isLoading: boolean = false; // Loader property

  constructor(
    private beneficiaryComplianceHoldService: BeneficiaryComplianceHoldService,
    private router: Router
  ) {}

  ngOnInit() {
    this.getStatusRecords();
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  fetchBeneficiaryComplicance(): void {
    if (!this.beneficiaryNo) {
      alert('Please enter a beneficiary number.');
      return;
    }

    this.isLoading = true; // Show loader
    this.beneficiaryComplianceHoldService
      .getAllBeneficiaryComplicance(this.beneficiaryNo)
      .subscribe(
        (data) => {
          this.beneficiaryDetails = data;
          if (this.beneficiaryDetails.length === 0) {
            alert('No data found.');
          }
          this.isLoading = false; // Hide loader
        },
        (error) => {
          alert(error?.error?.message);
          this.beneficiaryDetails = [];
          this.isLoading = false; // Hide loader
        }
      );
  }

  navigateToBeneficiary(beneficiaryNumber: any) {
    this.router.navigate([
      `/panel/beneficiary-compliance-hold-view/${beneficiaryNumber}`,
    ]);
  }

  getOfacLogDetails(transactionNumber: string) {
    this.beneficiaryComplianceHoldService
      .complicanceOfacLogDetails(transactionNumber)
      .subscribe(
        (response: any) => {
          console.log('OFAC Log Details:', response);
        },
        (error) => {
          console.error('Error fetching OFAC Log Details:', error);
        }
      );
  }

  getStatusRecords() {
    this.isLoading = true; // Show loader
    this.beneficiaryComplianceHoldService
      .getAllBeneficiaryComplicance(this.beneficiaryNo)
      .subscribe(
        (response: any[]) => {
          this.beneficiaryDetails = response;
          if (this.beneficiaryDetails.length === 0) {
            // alert('No data found.');
          }
          this.isLoading = false; // Hide loader
        },
        (error) => {
          console.error('Error fetching beneficiary details:', error);
          this.isLoading = false; // Hide loader
        }
      );
  }
}
