import { Component, TemplateRef, ViewChild } from '@angular/core';
import { CancelPaidTransactionService } from '../services/cancel-paid-transaction.service';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { AuthService } from '../services/auth.service';
import { ViewTransactionStatusForAgentService } from '../view-transaction-status-for-agent.service';
import { ToastrService } from 'ngx-toastr';
import { getCacheData } from '../utils/helper';

@Component({
  selector: 'app-cancel-paid-transaction',
  templateUrl: './cancel-paid-transaction.component.html',
  styleUrls: ['./cancel-paid-transaction.component.css'],
})
export class CancelPaidTransactionComponent {
  @ViewChild('viewTransactionLog') viewTransactionLogModal;
  bsModalRef: BsModalRef;
  @ViewChild('viewTransactionReceipt') viewTransactionReceiptModal;
  bsTransactionReceiptModalRef: BsModalRef;

  agentAccountList: any = [];
  selectedAgent: any = 0;
  agentBranchList: any = [];
  selectedAgentBranch: any;
  isLoading: boolean = false;
  receiptData: any = {};

  cancelledTypeList: any = [
    {
      id: 0,
      type: 'All',
    },
    {
      id: 10,
      type: 'CancelWithOutCharges',
    },
    {
      id: 11,
      type: 'CancelWithCharges',
    },
  ];
  selectedCancelledType: any = 0;
  statusList: any = [
    {
      id: -1,
      type: 'All',
    },
    {
      id: 0,
      type: 'Pending',
    },
    {
      id: 1,
      type: 'Processed',
    },
  ];
  selectedStatus: any = -1;
  fromDate: any = '';
  toDate: any = '';
  transactionNo: any = '';
  allCancelPaidTransactions: any = [];
  transactionsLogList: any = [];
  tableCancelledType: any = [
    {
      id: 10,
      type: 'Cancellation / Refund charges',
    },
    {
      id: 11,
      type: 'Cancellation / do not refund charges',
    },
  ];
  selectedCancelledTypeList: any = 10;

  constructor(
    private cancelPaidSvc: CancelPaidTransactionService,
    private modalService: BsModalService,
    private ViewTransactionStatusForAgentSvc: ViewTransactionStatusForAgentService,
    private authSvc: AuthService,
    private toastrSvc: ToastrService
  ) {}

  ngOnInit(): void {
    this.getAgentAcountList();
    this.getAgentBranchList(0);
    this.fromDate = this.authSvc.getTodayDate();
    this.toDate = this.authSvc.getTodayDate();
  }

  getAgentAcountList() {
    this.cancelPaidSvc.getAgentAcountList().subscribe({
      next: (res: any) => {
        this.agentAccountList = res;
      },
      error: (err) => {
        this.agentAccountList = [];
      },
      complete: () => {},
    });
  }

  onAgentAccountChange(agentId: any) {
    if (agentId != 0) {
      this.getAgentBranchList(agentId);
    }
  }

  getAgentBranchList(agentId: any = 0) {
    this.cancelPaidSvc.getAgentBranchList(agentId).subscribe({
      next: (res: any) => {
        this.agentBranchList = res;
        this.selectedAgentBranch = res?.[0]?.ID;
      },
      error: (err) => {
        this.agentBranchList = [];
      },
      complete: () => {},
    });
  }

  displayByDate() {
    if (!this.fromDate || !this.toDate) {
      alert('Please Select From and To Dates.');
      return;
    }
    this.transactionNo = '';
    this.isLoading = true;
    this.cancelPaidSvc
      .getAllRecordByStartAndEndDates(
        this.selectedAgent,
        this.selectedAgentBranch,
        this.selectedCancelledType,
        this.selectedStatus,
        this.fromDate,
        this.toDate
      )
      .subscribe({
        next: (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.allCancelPaidTransactions = res.map((transaction: any) => ({
            ...transaction,
            cancelledType: 10,
          }));
        },
        error: () => {
          this.allCancelPaidTransactions = [];
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
  }

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Fill Transaction Number!');
      return;
    }
    this.isLoading = true;

    this.cancelPaidSvc
      .getRecordByTransactionNumber(0, this.transactionNo)
      .subscribe({
        next: (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            return;
          }
          this.allCancelPaidTransactions = res.map((transaction: any) => ({
            ...transaction,
            cancelledType: 10,
          }));
        },
        error: () => {
          this.allCancelPaidTransactions = [];
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
  }

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template, { class: 'modal-lg' });
  }

  openTransactionLogModal(transactionNo: any) {
    if (transactionNo) {
      this.getTransactionLogByTransactionNo(transactionNo);
    }
  }

  getTransactionLogByTransactionNo(transactionNumber: any = 0) {
    this.cancelPaidSvc
      .getTransactionLogByTransactionNo(transactionNumber)
      .subscribe({
        next: (res: any) => {
          this.transactionsLogList = res;
          this.openModal(this.viewTransactionLogModal);
        },
        error: () => {
          this.transactionsLogList = [];
        },
        complete: () => {},
      });
  }

  cancelTransaction(transaction: any) {
    const payload = {
      AppID: transaction?.App_ID,
      TransactionNumber: transaction?.TransactionNumber,
      TrStatusID: transaction?.cancelledType,
      Remark: transaction?.Remark ? transaction?.Remark : '',
      UserID: getCacheData('G_UserID'),
    };
    this.cancelPaidSvc.cancelTransaction(payload).subscribe({
      next: (res: any) => {
        if (this.transactionNo) {
          this.filterByTransactioNo();
        } else {
          this.displayByDate();
        }
      },
      error: (err) => {
        this.toastrSvc.error(err?.error?.message);
      },
      complete: () => {},
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  getTransactionNumberDetail(item) {
    const trnNo = item?.TransactionNumber;
    this.ViewTransactionStatusForAgentSvc.getTransactionReport(trnNo).subscribe(
      (res: any) => {
        if (res?.[0]?.IsDuplicate == true) {
          alert('No data found.');
          return;
        }
        this.receiptData = res?.[0];
        this.openTransactionReceiptModal(this.viewTransactionReceiptModal);
      }
    );
  }

  openTransactionReceiptModal(template: TemplateRef<any>) {
    this.bsTransactionReceiptModalRef = this.modalService.show(template, {
      class: 'modal-lg',
    });
  }
}
