import { Component, ViewChild, ElementRef } from '@angular/core';
import { DailyRemittanceService } from '../services/daily-remittance.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-daily-remittance-dashboard-report',
  templateUrl: './daily-remittance-dashboard-report.component.html',
  styleUrls: ['./daily-remittance-dashboard-report.component.css'],
})
export class DailyRemittanceDashboardReportComponent {
  @ViewChild('reportTable', { static: false }) reportTable: ElementRef;
  
  selectedFileFormat: any = '1';
  showTable: boolean = false;
  customerType: string = '1';  // Default value
  // endDate: string = '';  // Variable to store the selected date
  endDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  reportData: any[] = [];  // Variable to store the fetched data

  isLoading: boolean = false;
  constructor(private dailyRemittanceService: DailyRemittanceService) {}

  onSubmit(): void {
    if (!this.endDate) {
      alert('Please select a date.');
      return;
    }
    this.isLoading=true
    this.dailyRemittanceService.getDashboardReportList(this.endDate, this.customerType).subscribe(
      (data: any) => {
        console.log(data);
        this.isLoading=false
        this.reportData = data;  // Store the fetched data
        this.showTable = true;
           // Show the table after data is fetched
      },
      (error) => {
        console.error(error);
        // this.isLoading=false   // Show the table after data is fetched
      }
    );
  }
  exportToExcel(): void {
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(this.reportTable.nativeElement);
  
    // Adjust date format if needed
    XLSX.utils.sheet_add_json(ws, this.reportData, {
      header: [
        "RegDateTime", "CustomerNo", "CustomerAccountNo", "FirstName", "LastName", 
        "MiddleName", "Mobile", "Nationality", "Total_Txn", "Active_Beneficiaries", 
        "Deactive_Beneficiaries", "Total_No_Of_Beneficiaries", "Txn_Last_3_Months", 
        "FirstTxnDate", "FirstTxnNumber", "MostRecentTxnDate", "MostRecentTxnNumber", 
        "SecondTxnDate", "SecondTxnNumber", "Channel"
      ],
      dateNF: 'yyyy-mm-dd' // Date format for Excel
    });
  
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');
  
    const fileName = `Report_${Date.now()}.xlsx`;
    this.isLoading = true;
  
    setTimeout(() => {
      XLSX.writeFile(wb, fileName);
      this.isLoading = false;
    }, 1000);
  }
  


  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
