import { Component, OnInit } from '@angular/core';
import { ForexTreasuryService } from '../services/forex-treasury.service';

@Component({
  selector: 'app-forex-treasury',
  templateUrl: './forex-treasury.component.html',
  styleUrls: ['./forex-treasury.component.css']
})
export class ForexTreasuryComponent implements OnInit {
  reports: any[];
  countries: any[];
  agents: any[];

  selectedReportType: string = 'detailed';  // Default to "Detailed Report"
  selectedCriteria: string = '8';   // Default to "Sending Agents Wise"
  selectedAgent: number ;   // Default to "Sending Agents Wise"
  selectedReports: string;


  gridData: any[] = [];

  selectedCountryId: number;

  fromDate: string;
  toDate: string;

  constructor(private forexService: ForexTreasuryService) { }

  ngOnInit(): void {
    this.initializeDates();
    this.loadReports();
    this.loadCountries();
    this.selectedReports = 'Income'; // Initialize selectedReports
  }

  initializeDates(): void {
    const today = new Date();
    const yyyy = today.getFullYear();
    const mm = String(today.getMonth() + 1).padStart(2, '0'); // Months are zero-based
    const dd = String(today.getDate()).padStart(2, '0');

    this.fromDate = `${yyyy}-${mm}-${dd}`;
    this.toDate = `${yyyy}-${mm}-${dd}`;
  }

  loadReports() {
    this.forexService.getReports().subscribe(reports => {
      this.reports = reports;
    });
  }

  loadCountries() {
    this.forexService.getCountries().subscribe(countries => {
      this.countries = countries;
    });
  }

  onCountryChange(countryId: number) {
    this.selectedCountryId = countryId;
    this.loadAgents(countryId);
  }

  loadAgents(countryId: number) {
    this.forexService.getAgents(countryId).subscribe(agents => {
      this.agents = agents;
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
  onAgentChange(event: any) {
    this.selectedAgent = event.target.value;
    console.log('Selected Agent ID:', this.selectedAgent);
    // Op
  }
  onReportChange(event: any) {
    this.selectedReports = event.target.value; // Update selectedReports on change
    console.log('Selected Report:', this.selectedReports);
    // You can perform additional actions here based on the selected report
  }
  viewDetails() {
    const detailedReport = this.selectedReportType === 'detailed';
    this.forexService.getGridData(this.selectedReports, this.selectedAgent, this.fromDate, detailedReport,this.selectedCriteria)
      .subscribe(data => {
        if (data.length === 0) {
          alert('No data found');
        }
        this.gridData = data;
      }, error => {
        alert(error.error.message
        );
        console.error('Error fetching grid data:', error);
      });
  }
}
