import { Component, ViewChild, TemplateRef } from '@angular/core';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { ManageAgencyUsersService } from '../manage-agency-users.service';
import { FormControl, FormGroup, Validators } from '@angular/forms';

@Component({
  selector: 'app-manage-agency-users',
  templateUrl: './manage-agency-users.component.html',
  styleUrls: ['./manage-agency-users.component.css'],
})
export class ManageAgencyUsersComponent {
  bsModalRef: BsModalRef;
  @ViewChild('manageagencyuser') manageagencyuser: TemplateRef<any>;
  @ViewChild('setDetails') setDetailsTemplate: TemplateRef<any>;
  countryList: any;
  groupList: any;
  countryID: any;
  isLoading: boolean = false;

  groupID: any;
  userID: any;
  userList: any;
  gridGroupList: any;
  allRecordList: any;
  subgridGroupList: any;
  addCountryList: any;
  AgentTypeID: any;
  addGroupList: any;
  setAgencyUserDetails: any = [];
  agencyUserBranch: any = {};
  selectedCountry: any = '0';
  selectedGroup: any = '-2';
  selectedUser: any = '-2';

  ManageAgencyUsersForm = new FormGroup({
    UserName: new FormControl('', Validators.required),
    EmailID: new FormControl('', Validators.required),
    UserNameID: new FormControl('', Validators.required),
    GroupID: new FormControl(0, Validators.required),
    UserId: new FormControl(0, Validators.required),
    CountryID: new FormControl(0, Validators.required),
    SubUserGroup_Id: new FormControl('', Validators.required),
    AgentType: new FormControl('None', Validators.required),
  });
  constructor(
    private modalService: BsModalService,
    private ManageAgencyUsersSvc: ManageAgencyUsersService
  ) {}

  ngOnInit(): void {
    this.getCountry();
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
  getDetails() {
    if (!this.f.UserNameID) {
      alert('Please Enter UserName or Id.');
      return;
    }
    this.ManageAgencyUsersSvc.getDetails(String(this.f.UserNameID)).subscribe(
      (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allRecordList = res[0].manageAgencyUserResponse;
        this.gridGroup(
          this.allRecordList[0].Agent_ID,
          this.allRecordList[0].User_Group
        );
        this.subGridGroup(this.allRecordList[0].User_Group);
      }
    );
  }

  openAgencyUserModal() {
    this.bsModalRef = this.modalService?.show(this.manageagencyuser, {
      class: 'modal-lg',
    });
  }
  getCountry() {
    this.ManageAgencyUsersSvc.getCountry().subscribe((res) => {
      this.countryList = res;
    });
  }
  onSelectCountry($event) {
    this.countryID = $event.target.value;
    this.getGroup(this.countryID);
  }

  getGroup(CountryID: any) {
    this.ManageAgencyUsersSvc.getGroup(CountryID).subscribe((res) => {
      this.groupList = res;
      if (this.groupList?.length > 0) {
        this.selectedGroup = res?.[0]?.SubUserGroup_Id;
        this.getUser(this.countryID, this.selectedGroup);
        if (!this.f.UserNameID) {
          this.subGridGroup(1);
        }
      } else {
        this.selectedGroup = '-2';
        this.selectedUser = '-2';
        this.groupList = [];
        this.userList = [];
      }
    });
  }

  onSelectGroup($event) {
    this.groupID = $event.target.value;
    this.getUser(this.countryID, this.groupID);
  }

  getUser(CountryID: any, GroupID: any) {
    this.ManageAgencyUsersSvc.getUser(CountryID, GroupID).subscribe((res) => {
      this.userList = res;
      this.selectedUser = '0';
    });
  }

  onSelectUserID($event) {
    this.userID = $event.target.value;
    console.log('this.userID: ', this.userID);
  }

  getAllRecord() {
    if (this.selectedCountry == '0') {
      alert('Please Select Country.');
      return;
    }

    this.isLoading = true;

    this.ManageAgencyUsersSvc.getAllRecord(
      this.selectedGroup,
      this.selectedUser
    ).subscribe({
      next: (res: any) => {
        if (res?.length == 0) {
          alert('No data found.');
          return;
        }
        this.allRecordList = res;
        console.log('this.allRecordList: ', this.allRecordList);
        // this.gridGroup(res[0].Agent_ID, res[0].User_Group);
        // this.subGridGroup(res[0].User_Group);
        this.isLoading = false;
      },
      error: (err) => {
        this.allRecordList = [];
        this.isLoading = false;
      },
      complete: () => {
        this.isLoading = false;
      },
    });
  }

  gridGroup(AgentID: any, User_Group: any) {
    this.ManageAgencyUsersSvc.gridGroup(AgentID, User_Group).subscribe(
      (res) => {
        this.gridGroupList = res;
      }
    );
  }

  subGridGroup(User_Group: any) {
    this.ManageAgencyUsersSvc.subGridGroup(User_Group).subscribe((res) => {
      this.subgridGroupList = res;
    });
  }

  addCountry() {
    this.ManageAgencyUsersSvc.addCountry().subscribe((res) => {
      this.addCountryList = res;
    });
  }

  onSelectAddCountry(event) {
    this.addGroup(this.AgentTypeID, event);
  }

  onSelectType($event) {
    this.addCountry();
    this.AgentTypeID = $event.target.value;
  }

  addGroup(AgentTypeID: any = 1, CountryID) {
    this.ManageAgencyUsersSvc.addGroup(AgentTypeID, CountryID).subscribe(
      (res) => {
        this.addGroupList = res;
      }
    );
  }

  isDeleted() {
    return this.allRecordList
      .filter((con) => con.isDeleted)
      .map((con) => ({
        UserName: String(con.User_Name),
        IschkDelete: con.isDeleted,
        UserId: String(con.UserId),
      }));
  }

  isUpdated() {
    return this.allRecordList
      .filter((con) => !con.isDeleted)
      .map((con) => ({
        UserName: String(con.User_Name),
        EmailID: String(con.EmailID),
        UserGroup: con.User_Group,
        SubUserGroup: con.SubUserGroup_Id,
        locked: String(con.Locked),
        firstlogin: String(con.FirstLogin),
        WebLogin: String(con.AllowWebLogin),
        UserId: String(con.UserId),
        IsChkReset: true,
        G_UserID: String(con.User_Group),
      }));
  }

  update() {
    this.allRecordList
      .filter((con) => con.isDeleted)
      .map((con) => {
        const payload = this.isDeleted();
        this.ManageAgencyUsersSvc.Delete(payload).subscribe((res) => {
          res;
        });
      });

    const payload = this.isUpdated();
    this.ManageAgencyUsersSvc.update(payload).subscribe((res) => {
      res;
    });
  }

  get f() {
    return this.ManageAgencyUsersForm.value;
  }

  addUser() {
    const payload = {
      UserName: this.f.UserName,
      AgentType: this.f.AgentType,
      GroupID: this.f.SubUserGroup_Id,
      G_Branch_ID: 1,
      EmailID: this.f.EmailID,
      G_UserID: '1',
    };

    this.ManageAgencyUsersSvc.addAgencyUser(payload).subscribe((res) => {
      res;
    });

    this.bsModalRef.hide();
  }

  openSetDetailsModal() {
    this.bsModalRef = this.modalService.show(this.setDetailsTemplate, {
      class: 'modal-xl',
    });
  }

  setAgencyUser(agencyUser: any) {
    this.isLoading = true;
    if (agencyUser) {
      this.ManageAgencyUsersSvc.set(
        agencyUser?.UserId,
        agencyUser?.User_Group,
        agencyUser?.SubUserGroup_Id
      ).subscribe({
        next: (res: any) => {
          this.setAgencyUserDetails = res;
          this.agencyUserBranch = res?.[0]?.manageAgencyUserBranchResponse?.[0];
          this.openSetDetailsModal();
        },
        error: (err) => {
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
    }
  }

  setTree() {
    // let payload;
    // payload = [
    //   {
    //     GroupID:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Group_Id,
    //     SubUserGroupID:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].SubUserGroup_Id,
    //     FourthFunctionID:
    //       this.setAgencyUserDetails?.[0]?.manageAgencyUserPopulateGroupResponse?.[0].FourthFunctionID?.toString(),
    //     ThirdFunctionID:
    //       this.setAgencyUserDetails?.[0]?.manageAgencyUserPopulateGroupResponse?.[0].ThirdFunctionID?.toString(),
    //     SecondFunctionID:
    //       this.setAgencyUserDetails?.[0]?.manageAgencyUserPopulateGroupResponse?.[0].SecondFunctionID?.toString(),
    //     FirstFunctionID:
    //       this.setAgencyUserDetails?.[0]?.manageAgencyUserPopulateGroupResponse?.[0].FirstFunctionID?.toString(),
    //     IschkLevel1:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].FirstActive,
    //     IschkLevel1Visible: true,
    //     IschkLevel2:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].SecondActive,
    //     IschkLevel2Visible: true,
    //     IschkLevel3:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].ThirdActive,
    //     IschkLevel3Visible: true,
    //     IschkLevel4:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].FourthActive,
    //     IschkLevel4Visible: true,
    //     IschkAccess_Add:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Access_Add,
    //     IschkAccess_AddVisible:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Add_IsVisible,
    //     IschkAccess_Edit:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Access_Edit,
    //     IschkAccess_EditVisible:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Edit_IsVisible,
    //     IschkAccess_Delete:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Access_Delete,
    //     IschkAccess_DeleteVisible:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Delete_IsVisible,
    //     IschkAccess_View:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Access_View,
    //     IschkAccess_ViewVisible:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].View_IsVisible,
    //     IschkAccess_Approve:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Approve_IsVisible,
    //     IschkAccess_ApproveVisible:
    //       this.setAgencyUserDetails?.[0]
    //         ?.manageAgencyUserPopulateGroupResponse?.[0].Access_Approve,
    //     FunctionID:
    //       this.setAgencyUserDetails?.[0]?.manageAgencyUserPopulateGroupResponse?.[0].FunctionID?.toString(),
    //     UserID: this.agencyUserBranch?.UserID?.toString(),
    //     QsUserID: '1',
    //   },
    // ];
    const payload = this.createPayload(
      this.setAgencyUserDetails,
      this.agencyUserBranch
    );

    this.ManageAgencyUsersSvc.setTree(payload).subscribe({
      next: (res: any) => {
        this.bsModalRef.hide();
        if (res) {
          alert('Updated successfully.');
          window.location.reload();
          return;
        }
      },
      error: (err) => {},
      complete: () => {
        this.bsModalRef.hide();
      },
    });
  }

  createPayload(setAgencyUserDetails: any, agencyUserBranch: any): any[] {
    const responses =
      setAgencyUserDetails?.[0]?.manageAgencyUserPopulateGroupResponse;
    if (!responses || !Array.isArray(responses)) {
      return [];
    }

    return responses.map((response: any) => ({
      GroupID: response.Group_Id,
      SubUserGroupID: response.SubUserGroup_Id,
      FourthFunctionID: response.FourthFunctionID?.toString()
        ? response.FourthFunctionID?.toString()
        : '',
      ThirdFunctionID: response.ThirdFunctionID?.toString()
        ? response.ThirdFunctionID?.toString()
        : '',
      SecondFunctionID: response.SecondFunctionID?.toString()
        ? response.SecondFunctionID?.toString()
        : '',
      FirstFunctionID: response.FirstFunctionID?.toString()
        ? response.FirstFunctionID?.toString()
        : '',
      IschkLevel1: response.FirstActive,
      IschkLevel1Visible: true,
      IschkLevel2: response.SecondActive,
      IschkLevel2Visible: true,
      IschkLevel3: response.ThirdActive,
      IschkLevel3Visible: true,
      IschkLevel4: response.FourthActive,
      IschkLevel4Visible: true,
      IschkAccess_Add: response.Access_Add,
      IschkAccess_AddVisible: response.Access_Add,
      IschkAccess_Edit: response.Access_Edit,
      IschkAccess_EditVisible: response.Access_Edit,
      IschkAccess_Delete: response.Access_Delete,
      IschkAccess_DeleteVisible: response.Access_Delete,
      IschkAccess_View: response.Access_View,
      IschkAccess_ViewVisible: response.Access_View,
      IschkAccess_Approve: response.Access_Approve,
      IschkAccess_ApproveVisible: response.Approve_IsVisible,
      FunctionID: response.FunctionID?.toString(),
      UserID: agencyUserBranch?.UserID?.toString(),
      QsUserID: '1',
    }));
  }
}
