import { Component, OnInit, TemplateRef, ViewChild } from '@angular/core';
import { ManageCurrencyService } from '../services/manage-currency.service';
import { getCacheData } from '../utils/helper';
import { BsModalRef, BsModalService } from 'ngx-bootstrap/modal';
import { FormGroup, FormControl, Validators,AbstractControl, ValidationErrors } from '@angular/forms';

import { ToastrService } from 'ngx-toastr';

@Component({
  selector: 'app-manage-currency',
  templateUrl: './manage-currency.component.html',
  styleUrls: ['./manage-currency.component.css'],
})
export class ManageCurrencyComponent implements OnInit {
  @ViewChild('manageCurrency') manageCurrencyModal;
  bsModalRef: BsModalRef;

  currencies: any = [];
  isLoading: boolean = false;

  type: any = 'add';

  allCurrencies: any = [];
  selectedCurrency = 'All';

  currencyTypes: any = [
    {
      id: 2,
      name: 'Sending',
    },
    {
      id: 3,
      name: 'Receiving',
    },
    {
      id: 4,
      name: 'Send/Receive',
    },
  ];

  manageCurrencyForm = new FormGroup({
    currency_id: new FormControl(),
    currency_name: new FormControl('', [
      Validators.required,
      Validators.pattern(/^[A-Za-z ]+$/)  // Allows only letters and spaces
    ]),
    currency_short_name: new FormControl('', [
      Validators.required,
      Validators.pattern(/^[A-Za-z0-9]+$/)  // Allows alphanumeric characters without spaces
    ]),
    scale: new FormControl('', [
      Validators.required,
      Validators.min(2),
      Validators.max(4)
    ]),
    active: new FormControl(false),
    is_settlement_currency: new FormControl(false),
    entered_by: new FormControl(''),
    currency_type: new FormControl('Select Currency Type', Validators.required),
  });

  
  constructor(
    private currencyService: ManageCurrencyService,
    private modalService: BsModalService,
    private toastrSvc: ToastrService

  ) {}

  ngOnInit(): void {
    this.getAllCurrencies();
  }
  forbiddenNumbersValidator(control: AbstractControl): ValidationErrors | null {
    const hasNumber = /\d/.test(control.value);
    return hasNumber ? { 'forbiddenNumbers': true } : null;
  }
  getAllCurrencies(currency_id: any = 0) {
    this.isLoading = true;
  
    this.currencyService
      .getCurrencies(
        getCacheData('admin_token'),
        getCacheData('username'),
        currency_id
      )
      .subscribe({
        next: (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            this.allCurrencies = []; // Ensure dropdown is cleared if no data is found
            this.isLoading = false;
            return;
          }
          this.currencies = [...res]; // This might be used for other purposes, ensure it's set
          this.allCurrencies = [...res];  // Update dropdown with new data
          this.isLoading = false;
        },
        error: (err) => {
          console.error('Failed to load currencies:', err);
          this.currencies = [];
          this.allCurrencies = []; // Clear dropdown if there is an error fetching data
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
  }
  getAllCurrenciesbyid(currency_id: any = 0) {
    this.isLoading = true;
  
    this.currencyService
      .getCurrencies(
        getCacheData('admin_token'),
        getCacheData('username'),
        currency_id
      )
      .subscribe({
        next: (res: any) => {
          if (res?.length == 0) {
            alert('No data found.');
            this.allCurrencies = []; // Ensure dropdown is cleared if no data is found
            this.isLoading = false;
            return;
          }
          this.currencies = [...res]; // This might be used for other purposes, ensure it's set
        
          this.isLoading = false;
        },
        error: (err) => {
          console.error('Failed to load currencies:', err);
          this.currencies = [];
          this.allCurrencies = []; // Clear dropdown if there is an error fetching data
          this.isLoading = false;
        },
        complete: () => {
          this.isLoading = false;
        },
      });
  }
  

  openModal(template: TemplateRef<any>) {
    this.bsModalRef = this.modalService.show(template);
  }

  openCurrencyModal(type: any) {
    this.type = type;
    if (this.type == 'add') {
      this.manageCurrencyForm.reset({
        currency_type: 'Select Currency Type',
      });
    }
    this.openModal(this.manageCurrencyModal);
  }

  get f() {
    return this.manageCurrencyForm.value;
  }

  addCurrency() {
    console.log(this.manageCurrencyForm, "d");
    if (!this.manageCurrencyForm.valid) {
      alert('Please Fill Required Fields!');
      return;
    }
  
    if (this.f.currency_type == 'Select Currency Type') {
      alert('Please Select Currency Type!');
      return;
    }
  
    let payload;
    payload = {
      CurrencyName: this.f.currency_name,
      CurrencyShortName: this.f.currency_short_name,
      Scale: this.f.scale,
      Active: this.f.active,
      IsSettlementCurrency: this.f.is_settlement_currency,
      EnteredBy: this.f.entered_by || 'none',
      CurrencyType: this.f.currency_type,
    };
  
    const currency_type = this.currencyTypes.find(
      (c) => c.name == this.f.currency_type
    );
  
    if (currency_type) {
      payload = {
        ...payload,
        CurrencyType: currency_type?.id,
      };
    }
  
    if (this.type === 'edit') {
      payload = {
        ...payload,
        CurrencyId: this.f.currency_id,
      };
    }
  
    const serviceMethod =
      this.type === 'add' ? 'addCurrency' : 'updateCurrency';
  
    this.currencyService[serviceMethod](
      getCacheData('admin_token'),
      getCacheData('username'),
      payload
    ).subscribe({
      next: (res: any) => {
        this.getAllCurrencies();
        this.modalService.hide();
        const message = this.type === 'add' ? 'Currency added successfully!' : 'Currency updated successfully!';
        // alert(res);
        this.toastrSvc.success(res)

        this.manageCurrencyForm.reset({
          currency_type: 'Select Currency Type',
        });
      },
      error: (err) => {
        // console.error(err);
        // const errorMessage = err.error?.message || 'An error occurred while processing the currency.';
        // alert(errorMessage);
        this.toastrSvc.error(err.error?.message)
      },
      complete: () => {},
    });
  }
  

  editCurrency(currency: any) {
    this.manageCurrencyForm.patchValue({
      currency_id: currency?.ID,
      currency_name: currency?.CurrencyName,
      currency_short_name: currency?.CurrencyShortName,
      currency_type: currency?.CurrencyType,
      scale: currency?.Scale,
      active: currency?.Active,
      is_settlement_currency: currency?.IsSettlementCurrency,
      entered_by: currency?.EnteredBy,
    });
    this.openCurrencyModal('edit');

  }

  searchCurrency() {
    if (this.selectedCurrency != 'All') {
      this.getAllCurrenciesbyid(this.selectedCurrency);
    } else {
      this.getAllCurrencies();
    }
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
  toggleFullName(columnName: string, event: Event) {
    const target = event.target as HTMLElement;
    if (target.textContent === columnName) {
      target.textContent = columnName.slice(0, 3) + '...';
    } else {
      target.textContent = columnName;
    }
  }
  transformToUpperCase(event: Event, controlName: string): void {
    const input = event.target as HTMLInputElement;
    input.value = input.value.toUpperCase();
    this.manageCurrencyForm.get(controlName)?.setValue(input.value);
  }
  
  
}
