import { Component } from '@angular/core';
import { ManageUserSubgroupService } from '../manage-user-subgroup.service';

@Component({
  selector: 'app-manage-user-subgroups',
  templateUrl: './manage-user-subgroups.component.html',
  styleUrls: ['./manage-user-subgroups.component.css'],
})
export class ManageUserSubgroupsComponent {
  groupList: any;
  subGroupList: any;
  GroupId: any;
  subGroupId: any;
  accessList: any = [];
  hierarchicalFunctions: any = [];

  constructor(private ManageUserSubgroupsSvc: ManageUserSubgroupService) {}
  ngOnInit(): void {
    this.getGroup();
  }

  getGroup() {
    this.ManageUserSubgroupsSvc.getGroup().subscribe({
      next: (res: any) => {
        this.groupList = res;
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  onSelectGroup(GroupId) {
    this.GroupId = GroupId;
    this.getSubGroup(GroupId);
  }

  getSubGroup(GroupId) {
    this.ManageUserSubgroupsSvc.getSubGroup(GroupId).subscribe({
      next: (res: any) => {
        this.subGroupList = res;
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
  onSelectSubGroup(subGroupId) {
    this.subGroupId = subGroupId;
  }

  getAccess() {
    if (!this.GroupId) {
      alert('Please Select Group.');
      return;
    }
    if (!this.subGroupId) {
      alert('Please Select User.');
      return;
    }
    this.ManageUserSubgroupsSvc.getAccess(
      this.GroupId,
      this.subGroupId
    ).subscribe({
      next: (res: any) => {
        this.accessList = res[0].SubGroupList;
        this.hierarchicalFunctions = this.buildHierarchy(this.accessList);
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  updateChildren(node: any) {
    const updateChildNodes = (parentNode: any) => {
      if (parentNode.children && parentNode.children.length) {
        parentNode.children.forEach((childNode: any) => {
          // Set child nodes to the same state as the parent node
          childNode.IsChecked = parentNode.IsChecked;

          // Recursively call to handle further levels
          updateChildNodes(childNode);
        });
      }
    };

    // Update child nodes based on the current state of the parent node
    updateChildNodes(node);
  }

  isUpdated() {
    const updatedFunctions = [];

    const traverse = (nodes: any[]) => {
      nodes.forEach((node) => {
        if (node.IsChecked) {
          updatedFunctions.push({
            GroupID: Number(this.GroupId),
            SubUserGroupID: Number(this.subGroupId),
            FunctionID: Number(node.FunctionID),
            Status: String(node.IsChecked),
          });
        }
        if (node.children && node.children.length) {
          traverse(node.children);
        }
      });
    };

    traverse(this.hierarchicalFunctions);
    return updatedFunctions;
  }

  setAccess() {
    // const payload = this.isUpdated();
    const transform = (array) => {
      const result = [];

      const traverse = (nodes, groupID, subUserGroupID) => {
        nodes.forEach((node) => {
          result.push({
            GroupID: Number(this.GroupId),
            SubUserGroupID: Number(this.subGroupId),
            FunctionID: Number(node.FunctionID),
            Status: node.IsChecked ? 'true' : 'false',
          });

          if (node.children && node.children.length > 0) {
            traverse(node.children, node.FunctionID, 0); // Recursively traverse children
          }
        });
      };

      traverse(array, 0, 0); // Start traversing from the root level
      return result;
    };

    const payload = transform(this.hierarchicalFunctions);
    this.ManageUserSubgroupsSvc.setAccess(payload).subscribe({
      next: (res: any) => {
        alert('Records updated successfully.');
        window.location.reload();
        return;
      },
      error: (err) => {},
      complete: () => {},
    });
  }

  buildHierarchy(functions: any[]): any[] {
    const functionMap: { [key: string]: any } = {};
    const rootFunctions: any[] = [];

    functions.forEach((func) => {
      func.children = [];
      functionMap[func.FunctionName] = func;

      if (func.ParentName === '0') {
        rootFunctions.push(func);
      } else {
        const parentFunc = functionMap[func.ParentName];
        if (parentFunc) {
          parentFunc.children.push(func);
        }
      }
    });

    return rootFunctions;
  }
}
