import { Component, OnInit, ElementRef, ViewChild } from '@angular/core';
import { MgTransactionAccountDetailsService } from '../services/mg-transaction-account-details.service';
import * as XLSX from 'xlsx';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
 
@Component({
  selector: 'app-mg-transaction-account-details',
  templateUrl: './mg-transaction-account-details.component.html',
  styleUrls: ['./mg-transaction-account-details.component.css'],
})
export class MgTransactionAccountDetailsComponent implements OnInit {
  @ViewChild('transactionTable') transactionTable: ElementRef;
  @ViewChild('viewTransactionLog') viewTransactionLogModal;
 
  countryList: any = [];
  selectedCountryId: any;
  agentList: any = [];
  selectedAgentList: any = null;
  selectedFileFormat: string = '1';
  gridData: any[] = [];
  selectedCustomerType: string = '-1'; // Add this variable
  fromDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  transactionNo: string = ''; // Add this variable
  isLoading: boolean = false;
  transactionLogs: any[] = []; // Add this variable
  bsModalRef: BsModalRef; // Add this variable
 
  constructor(
    private mgTransactionService: MgTransactionAccountDetailsService,
    private modalService: BsModalService // Add this
  ) {}
 
  ngOnInit(): void {
    this.getCountryList();
    const currentDate = new Date().toISOString().split('T')[0];
    this.fromDate = currentDate;
    this.toDate = currentDate;
  }

  getCountryList() {
    this.mgTransactionService.getTransactionDetailsByCountry(0).subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryId =
          this.countryList.length > 0 ? this.countryList[0].Countryid : null;
        if (this.selectedCountryId !== null) {
          this.getAgentList(this.selectedCountryId);
        }
      },
      error: () => {
        this.countryList = [];
      },
    });
  }
 
  onCountryChange(countryId: any) {
    this.selectedCountryId = countryId;
    this.getAgentList(countryId);
  }
 
  getAgentList(countryId: any) {
    this.mgTransactionService
      .getTransactionDetailsByAgent(countryId)
      .subscribe({
        next: (res: any) => {
          this.agentList = res;
          this.selectedAgentList = -1;
        },
        error: () => {
          this.agentList = [];
        },
      });
  }
 
  onAgentChange(agentId: any) {}
 
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
  getTransactionDetailsGrid(byTxn:any): void {
    if (!this.fromDate || !this.toDate) {
      alert('Please select both From Date and To Date.');
      return;
    }
 
    this.isLoading = true;
    let valueToPass: string;
    if (byTxn === 1) {
      valueToPass = 'N';
    } else if (byTxn === 2) {
      valueToPass = 'Y';
    } else {
      valueToPass = byTxn; // Default to passing byTxn if it's not 1 or 2
    }
    this.mgTransactionService
      .getTransactionDetailsGrid(
        this.fromDate,
        this.toDate,
        parseInt(this.selectedCountryId),
        parseInt(this.selectedAgentList),
        this.selectedCustomerType,
        this.transactionNo,
        valueToPass
      )
      .subscribe(
        (response) => {
          this.gridData = response;
          this.isLoading = false; // Stop loading
        },
        (error) => {
          console.error('Error calling transaction details grid API:', error);
          this.isLoading = false; // Stop loading on error
        }
      );
  }
  exportToExcel() {
    const worksheet: XLSX.WorkSheet = XLSX.utils.json_to_sheet(this.gridData);
    const workbook: XLSX.WorkBook = {
      Sheets: { data: worksheet },
      SheetNames: ['data'],
    };
    XLSX.writeFile(workbook, 'TransactionDetails.xlsx');
  }
 
  onFromDateChange(event: any) {
    this.fromDate = event.target.value;
  }
 
  onToDateChange(event: any) {
    this.toDate = event.target.value;
  }
 
  openTransactionLogModal(transactionNumber: any) {
 
 
    this.mgTransactionService.getlogGrid(transactionNumber).subscribe(
      (data) => {
          // Assign the fetched data to this.transactionLogs
          this.transactionLogs = data;
 
          // // Show the modal after fetching the data
          // this.bsModalRef = this.modalService.show(this.viewTransactionLogModal, {
          //     class: 'modal-xl'
          // });
      },
      (error) => {
          // Handle error (optional)
          console.error('Error fetching transaction logs:', error);
      }
  );
   
    this.bsModalRef = this.modalService.show(this.viewTransactionLogModal, {
      class: 'modal-xl'
    });  }
}
 