import { Component, ElementRef, OnInit, ViewChild } from '@angular/core';
import { MonthlyFeesReconciliationService } from '../services/monthly-fees-reconciliation-report.service';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-monthly-fees-reconciliation-report',
  templateUrl: './monthly-fees-reconciliation-report.component.html',
  styleUrls: ['./monthly-fees-reconciliation-report.component.css'],
})
export class MonthlyFeesReconciliationReportComponent implements OnInit {
  @ViewChild('reportTable') reportTable: ElementRef;

  countries: any[] = [];
  selectedCountry: any='-1';
  selectedAgent: any='-1';
  selectedCurrency: any='-1';
  fromDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  toDate: string = new Date(new Date().toLocaleString("en-US", {timeZone: "Asia/Riyadh"})).toISOString().substring(0, 10);
  reportData: any[] = [];
  agents: any[] = [];
  currencies: any[] = [];
  selectedFileFormat: any = '1';
  isLoading: boolean = false; // Initialize isLoading to false

  constructor(private feesReconciliationService: MonthlyFeesReconciliationService) {}

  ngOnInit(): void {
    this.fetchCountries();
  }

  fetchCountries(): void {
    this.isLoading = true; // Set isLoading to true while fetching data
    this.feesReconciliationService.getMonthlyFeesReconciliation().subscribe(
      (data: any[]) => {
        this.countries = data;
      },
      (error) => {
        console.error(error);
      },
      () => {
        this.isLoading = false; // Set isLoading to false after data is fetched
      }
    );
  }

  onSubmit(): void {
    if (!this.fromDate || !this.toDate || !this.selectedCountry || !this.selectedAgent || !this.selectedCurrency) {
      alert('Please select all fields');
      return;
    }

    this.isLoading = true; // Set isLoading to true while fetching data
    this.feesReconciliationService
      .getMonthlyFeesReconciliationGrid(this.selectedCountry, this.selectedAgent, this.selectedCurrency, this.fromDate, this.toDate)
      .subscribe(
        (data: any[]) => {
         

          if (data?.length) {
            this.reportData = data;
          } else {
            alert('No data found.');
          }
        },
        (error) => {
          console.error(error);
          alert(error.error.message)
        },
        () => {
          this.isLoading = false; // Set isLoading to false after data is fetched
        }
      );
  }

  onCountryChange(): void {
    if (this.selectedCountry) {
      this.isLoading = true; // Set isLoading to true while fetching data
      this.feesReconciliationService.getMonthlyFeesReconciliationByAgent(this.selectedCountry).subscribe(
        (data: any[]) => {
          this.agents = data;
        },
        (error) => {
          console.error(error);
        },
        () => {
          this.isLoading = false; // Set isLoading to false after data is fetched
        }
      );
    }
  }

  onAgentChange(): void {
    if (this.selectedAgent) {
      this.isLoading = true; // Set isLoading to true while fetching data
      this.feesReconciliationService.getMonthlyFeesReconciliationByCurrency(this.selectedAgent).subscribe(
        (data: any[]) => {
          this.currencies = data;
        },
        (error) => {
          console.error(error);
        },
        () => {
          this.isLoading = false; // Set isLoading to false after data is fetched
        }
      );
    }
  }

  exportToExcel(): void {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(this.reportTable.nativeElement);

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Monthly_Fees_Reconciliation_Report${Date.now()}.xlsx`;

    this.isLoading = true; // Set isLoading to true while exporting
    setTimeout(() => { // Simulating export delay
      XLSX.writeFile(wb, fileName);
      this.isLoading = false; // Set isLoading to false after exporting
    }, 1000); // Example timeout, adjust as necessary for your actual export process
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }
}
