import { Component, OnInit } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { PayoutSummaryService } from '../services/payout-summary.service';

@Component({
  selector: 'app-payout-summary',
  templateUrl: './payout-summary.component.html',
  styleUrls: ['./payout-summary.component.css'],
})
export class PayoutSummaryComponent implements OnInit {
  agentAccounts: any[] = [];
  branches: any[] = [];
  selectedAgentAccount: any;
  selectedBranch: any;
  showTable: boolean = false;
  payoutData: any[] = [];
  payoutForm: FormGroup;
  formSubmitted: boolean = false;
  showValidationMessages: boolean = false;
  isLoading: boolean = false;

  constructor(
    private fb: FormBuilder,
    private payoutService: PayoutSummaryService
  ) {}

  ngOnInit(): void {
    this.loadAgent();
    this.initForm();
  }

  initForm(): void {
    const currentStartDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    const currentEndDate = new Date(new Date().toLocaleString("en-US", { timeZone: "Asia/Riyadh" })).toISOString().substring(0, 10);

    this.payoutForm = this.fb.group({
      startDate: [currentStartDate, Validators.required],
      endDate: [currentEndDate, Validators.required],
      selectedAgentAccount: [null, Validators.required],
      selectedBranch: [null, Validators.required],
    });
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  loadAgent(): void {
    this.payoutService.getAgentPayoutSummaryAgent(0).subscribe(
      (data: any[]) => {
        this.agentAccounts = data;
      },
      (error: any) => {
        console.error('Error fetching agent accounts:', error);
      }
    );
  }

  loadBranches(agentId: any): void {
    this.payoutService.getAgentPayoutSummaryBranch(agentId, 0, 0).subscribe(
      (data: any[]) => {
        this.branches = data;
      },
      (error: any) => {
        console.error('Error fetching branches:', error);
      }
    );
  }

  get f() {
    return this.payoutForm.value;
  }

  fetchData(): void {
    const { startDate, endDate, selectedAgentAccount, selectedBranch } =
      this.payoutForm.value;
    if (!startDate) {
      alert('Please Select Start Date.');
      return;
    }
    if (!endDate) {
      alert('Please Select End Date.');
      return;
    }
    if (!selectedAgentAccount) {
      alert('Please Select Agent Account.');
      return;
    }
    if (!selectedBranch) {
      alert('Please Select Branch.');
      return;
    }
    this.isLoading = true;
    this.payoutService
      .getAgentPayoutSummaryGrid(
        selectedAgentAccount,
        selectedBranch,
        startDate,
        endDate
      )
      .subscribe(
        (data: any[]) => {
          this.isLoading = false;
          if (data?.length == 0) {
            alert('No data found.');
            return;
          }
          if (data && data.length > 0) {
            this.payoutData = data[0].Table_1;
            this.showTable = true;
          } else {
            this.payoutData = [];
            this.showTable = false;
          }
        },
        (error: any) => {
          this.isLoading = false;
          console.error('Error fetching data:', error);
        }
      );
  }

  updateBranches(agentId: any): void {
    this.selectedAgentAccount = agentId;
    this.loadBranches(agentId);
  }

  onSubmit(): void {
    this.formSubmitted = true;
    if (this.payoutForm.valid) {
      this.fetchData();
    } else {
      this.showValidationMessages = true;
    }
  }

  isInvalid(controlName: string): boolean {
    const control = this.payoutForm.get(controlName);
    return control.invalid && (control.dirty || control.touched);
  }
}
