import { Component, OnInit } from '@angular/core';
import { RatesInSummarizedWayService } from '../services/rates-in-summarized-way.service';

@Component({
  selector: 'app-rates-summarized-way',
  templateUrl: './rates-summarized-way.component.html',
  styleUrls: ['./rates-summarized-way.component.css'],
})
export class RatesSummarizedWayComponent implements OnInit {
  wayReportDetails: any[] = [];
  countryList: any[] = [];
  agentList: any[] = [];
  receiveCountryList: any[] = [];
  receiveAgentList: any[] = [];
  selectedCountryId: any;
  isLoading: boolean = false;

  selectedAgentId: any = '-1';
  selectedReceiveCountryId: any = '-1';
  selectedReceiveAgentId: any = '-1';
  selectedReceiveCountry: any;
  selectedReceiveAgent: any;
  receiveCountries: any[] = [];
  receiveAgents: any[] = [];
  wayReportDetailsList: any[] = [];

  constructor(private ratesService: RatesInSummarizedWayService) {}

  ngOnInit(): void {
    this.getCountryData();
  }

  onAgentChange(agentId: any) {}

  getCountryData() {
    this.ratesService.getCountryData(0).subscribe({
      next: (res: any) => {
        this.countryList = res;
        this.selectedCountryId =
          this.countryList.length > 0 ? this.countryList[0].CountryID : null;
        const countryId = this.countryList.find(
          (c) => c.CountryName == 'SAUDI ARABIA'
        )?.CountryID;
        if (countryId) {
          this.selectedCountryId = countryId;
        } else {
          this.selectedCountryId =
            this.countryList.length > 0 ? this.countryList[0].CountryID : null;
        }

        if (this.selectedCountryId !== null) {
          this.loadAgentData(this.selectedCountryId);
          this.loadReceiveCountryData(this.selectedCountryId);
        }
      },
      error: () => {
        this.countryList = [];
      },
    });
  }

  loadAgentData(countryId: any) {
    this.ratesService.getSendAgentData(countryId).subscribe({
      next: (res: any) => {
        this.agentList = res;
        this.agentList.unshift({
          Agent_ID: 0,
          Agent_Name: 'ALL',
        });
        // this.selectedAgentId =
        //   this.agentList.length > 0 ? this.agentList[0].Agent_ID : null;
      },
      error: () => {
        this.agentList = [];
      },
    });
  }

  loadReceiveCountryData(countryId: any) {
    this.ratesService.getReceiveCountryData(countryId).subscribe({
      next: (res: any) => {
        this.receiveCountryList = res;
        this.receiveCountryList.unshift({
          CountryID: 0,
          CountryName: 'ALL',
        });
        this.loadReceiveAgentData(this.selectedReceiveCountryId);
        // this.selectedReceiveCountryId =
        //   this.receiveCountryList.length > 0
        //     ? this.receiveCountryList[0].CountryID
        //     : null;
        // if (this.selectedReceiveCountryId !== null) {
        // }
      },
      error: () => {
        this.receiveCountryList = [];
      },
    });
  }

  loadReceiveAgentData(countryId: any) {
    this.ratesService.getReceiveAgentData(countryId).subscribe({
      next: (res: any) => {
        this.receiveAgentList = res;
        this.receiveAgentList.unshift({
          Agent_ID: 0,
          Agent_Name: 'ALL',
        });
        // this.selectedReceiveAgentId =
        //   this.receiveAgentList.length > 0
        //     ? this.receiveAgentList[0].Agent_ID
        //     : null;
      },
      error: () => {
        this.receiveAgentList = [];
      },
    });
  }

  onCountryChange(countryId: any) {
    this.selectedCountryId = countryId;
    this.loadAgentData(countryId);
    this.loadReceiveCountryData(countryId);
  }

  onReceiveCountryChange(countryId: any) {
    if (countryId != '-1') {
      this.selectedReceiveCountryId = countryId;
      this.loadReceiveAgentData(countryId);
    }
  }
  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  callSummarizedWayReportCountry() {
    if (this.selectedAgentId == '-1') {
      alert('Please Select Send Agent.');
      return;
    }
    if (this.selectedReceiveCountryId == '-1') {
      alert('Please Select Receive Country.');
      return;
    }
    if (this.selectedReceiveAgentId == '-1') {
      alert('Please Select Receive Agent.');
      return;
    }
    this.isLoading = true;

    const requestBody = {
      CountryID: this.selectedCountryId, // Assuming selectedCountryId is obtained dynamically
      TargetCountryID: this.selectedReceiveCountryId, // Assuming selectedReceiveCountryId is obtained dynamically
      AgentID: this.selectedAgentId, // Assuming selectedAgentId is obtained dynamically
      IsAgent: true,
      TargetAgentID: this.selectedReceiveAgentId,
    };

    this.ratesService.SummarizedWayReportCountry(requestBody).subscribe(
      (response: any) => {
        this.wayReportDetailsList = response;
        this.isLoading = false;
      },
      (error) => {
        console.error('Error calling SummarizedWayReportCountry API:', error);
        this.isLoading = false;
      }
    );
  }
}
