import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { Observable } from 'rxjs';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class AgentReportsService {
  url = environment.apiUrl;

  constructor(private router: Router, private http: HttpClient) {}

  getCountries(countryType: string): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/agent-report/send-receive-trxn/country?CountryType=${countryType}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentReport(): Observable<any> {
    return this.http.get<any[]>(
      `${environment.apiUrl}/api/master/reports/agent-report/send-receive-trxn/select-report`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgents(countryId: number): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/agent-report/send-receive-trxn/agent?CountryID=${countryId}&SessionAgentID=0`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentBranches(agentId: number): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/agent-report/send-receive-trxn/agent-branch?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentReportCountry() {
    return this.http.get(
      `${environment.apiUrl}/api/master/reports/agent-report/agent-wise-trxn-sent-paid/country?UserID=0&Agent_Country_ID=0`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentReportCurrency(agentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/reports/agent-report/agent-wise-trxn-sent-paid/currency?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentList(countryId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/reports/agent-report/agent-wise-trxn-sent-paid/agent-list?CountryID=${countryId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentBranchList(agentId: any = 0, payload: any = {}) {
    return this.http.post(
      `${environment.apiUrl}/api/master/reports/agent-report/agent-wise-trxn-sent-paid/agent-branch?AgentID=${agentId}`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getUserList(agentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/master/reports/agent-report/agent-wise-trxn-sent-paid/user-list?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getSendReceiveTransaction(
    agentId: any = 0,
    fromDate: any = '',
    toDate: any = ''
  ) {
    return this.http.get(
      `${environment.apiUrl}/api/master/reports/agent-report/send-receive-trxn/send-receive-transaction?AgentID=${agentId}&FromDate=${fromDate}&ToDate=${toDate}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentTransactionSummary(
    agentId: any = 0,
    agentBranchId: any = 0,
    fromDate: any = '',
    toDate: any = ''
  ) {
    return this.http.get(
      `${environment.apiUrl}/api/master/reports/agent-report/send-receive-trxn/agent-transaction-summary?AgentID=${agentId}&AgentBranchID=${agentBranchId}&FromDate=${fromDate}&ToDate=${toDate}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getExchangeEarningReport(
    agentId: any = 0,
    agentBranchId: any = 0,
    fromDate: any = '',
    toDate: any = ''
  ) {
    return this.http.get(
      `${environment.apiUrl}/api/master/reports/agent-report/send-receive-trxn/exchange-earnings-report?AgentID=${agentId}&AgentBranchID=${agentBranchId}&FromDate=${fromDate}&ToDate=${toDate}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getCurrencies(): Observable<any> {
    return this.http.get<any[]>(
      `${this.url}/api/master/reports/agent-report/agent-wise-trxn-sent-paid/currency`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}
