import { HttpClient } from '@angular/common/http';
import { Injectable } from '@angular/core';
import { environment } from 'src/environments/environment';
import { getCacheData } from '../utils/helper';

@Injectable({
  providedIn: 'root',
})
export class CancelPaidTransactionService {
  url = environment.apiUrl;

  constructor(private http: HttpClient) {}

  getAgentAcountList() {
    return this.http.get(
      `${environment.apiUrl}/api/transaction-management/transaction-cancel-paid/agent-account?CountryID=0`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAgentBranchList(agentId: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/transaction-management/transaction-cancel-paid/agent-branch?AgentID=${agentId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAllRecord(
    agentId: any = 0,
    agentBranchId: any = 0,
    cancelledType: any = 0,
    statusId: any = -1
  ) {
    return this.http.get(
      `${environment.apiUrl}/api/transaction-management/transaction-cancel-paid/all-record?AgentID=${agentId}&SubGroupID=0&AgentBranchID=${agentBranchId}&CancelledType=${cancelledType}&StatusID=${statusId}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getAllRecordByStartAndEndDates(
    agentId: any = 0,
    agentBranchId: any = 0,
    cancelledType: any = 0,
    statusId: any = -1,
    startDate: any,
    endDate: any
  ) {
    return this.http.get(
      `${environment.apiUrl}/api/transaction-management/transaction-cancel-paid/all-record?AgentID=${agentId}&SubGroupID=0&AgentBranchID=${agentBranchId}&CancelledType=${cancelledType}&StatusID=${statusId}&StartDate=${startDate}&EndDate=${endDate}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getRecordByTransactionNumber(agentId: any = 0, transactionNumber: any = '') {
    return this.http.get(
      `${environment.apiUrl}/api/transaction-management/transaction-cancel-paid/transaction-number?AgentID=${agentId}&SubGroupID=0&TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  getTransactionLogByTransactionNo(transactionNumber: any = 0) {
    return this.http.get(
      `${environment.apiUrl}/api/transaction-management/transaction-status/log?TransactionNumber=${transactionNumber}`,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }

  cancelTransaction(payload: any = {}) {
    return this.http.post(
      `${environment.apiUrl}/api/transaction-management/transaction-cancel-paid/cancel`,
      payload,
      {
        headers: {
          Authorization: getCacheData('admin_token'),
          Accept: 'application/json',
          username: getCacheData('username'),
        },
      }
    );
  }
}
