import { Component, OnInit, ViewChild, TemplateRef, ElementRef } from '@angular/core';
import { ViewComplainceHoldListService } from '../services/view-complaince-hold-list.service';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { Router } from '@angular/router';
import { ChangeDetectorRef } from '@angular/core';
import * as XLSX from 'xlsx';

@Component({
  selector: 'app-view-complaince-hold-list',
  templateUrl: './view-complaince-hold-list.component.html',
  styleUrls: ['./view-complaince-hold-list.component.css'],
})
export class ViewComplainceHoldListComponent implements OnInit {
  @ViewChild('viewTransactionLog') viewTransactionLog: TemplateRef<any>;
  @ViewChild('viewCompliance') viewComplianceModal: TemplateRef<any>;
  @ViewChild('complianceTransactionTable') complianceTransactionTable: ElementRef;
  @ViewChild('complianceHoldTable') complianceHoldTable: ElementRef;
  bsModalRef: BsModalRef;
  records: any[] = [];
  complianceLogs: any;
  viewComplianceList: any = [];
  transactionNo: any = '';
  selectedRecord: any = -1;
  allComplianceTransactions: any = [];
  isLoading: boolean = false;
  selectedFileFormat = '1';
  transactionsLogList: any[] = [];

  allRecordList: any = [
    { id: -1, value: 'All' },
    { id: 10, value: '10 transactions' },
    { id: 25, value: '25 transactions' },
    { id: 50, value: '50 transactions' },
    { id: 100, value: '100 transactions' },
  ];

  constructor(
    private holdListService: ViewComplainceHoldListService,
    private modalService: BsModalService,
    private router: Router,
    private cdr: ChangeDetectorRef // Inject ChangeDetectorRef
  ) {}

  ngOnInit() {
    this.fetchData();
  }

  toggleSideBar() {
    const bodyElement = document.body;
    bodyElement.classList.toggle('sidebar-collapsein');
  }

  navigateToComplianceHoldTransaction(transactionNo: any) {
    this.router.navigate([`/panel/complaince-hold-transaction-process/${transactionNo}`]);
  }

  openComplianceHoldLogModal(transactionNo: any) {
    this.bsModalRef = this.modalService.show(this.viewTransactionLog, { class: 'modal-lg' });

    this.isLoading = true;
    this.holdListService.viewComplianceHoldListLog(transactionNo).subscribe(
      (data: any) => {
        console.log('Response from server:', data);
        this.transactionsLogList = data; // Assign data to transactionsLogList
        if (this.transactionsLogList.length === 0) {
          alert('No data found.');
        }
        this.isLoading = false;
        this.cdr.detectChanges(); // Manually trigger change detection
      },
      (error) => {
        console.error('Error fetching data:', error);
        this.isLoading = false;
      }
    );
  }

  closeComplianceHoldLogModal() {
    this.bsModalRef.hide();
  }

  fetchData() {
    this.isLoading = true;
    let topRecords: number;

    if (this.selectedRecord === 'All') {
      topRecords = 0;
    } else {
      topRecords = +this.selectedRecord;
    }

    this.holdListService.fetchComplianceHoldListRecords(topRecords).subscribe(
      (data: any[]) => {
        console.log('Fetched data:', data);
        this.records = data;
        if (this.records.length === 0) {
          alert('No data found.');
        }
        this.isLoading = false;
      },
      (error) => {
        console.error('Error fetching data:', error);
        this.isLoading = false;
      }
    );
  }

  fetchDataForSelectedRecord() {
    this.fetchData();
  }

  displayData() {
    this.fetchData();
  }

  openComplianceModal(transactionNumber: any) {
    this.bsModalRef = this.modalService.show(this.viewComplianceModal, { class: 'modal-lg' });

    this.isLoading = true;
    this.holdListService.viewComplianceHoldList(transactionNumber).subscribe(
      (response) => {
        console.log(response);
        this.viewComplianceList = response;
        if (this.viewComplianceList.length === 0) {
          alert('No data found.');
        }
        this.isLoading = false;
      },
      (error) => {
        console.error(error);
        this.isLoading = false;
      }
    );
  }

  filterByTransactioNo() {
    if (!this.transactionNo) {
      alert('Please Enter Transaction No!');
      return;
    }

    this.isLoading = true;
    this.holdListService.filterdByRecords(this.transactionNo).subscribe(
      (data: any[]) => {
        console.log('Filtered data:', data);
        this.records = data;
        if (this.records.length === 0) {
          alert('No data found.');
        }
        this.isLoading = false;
      },
      (error) => {
        console.error('Error fetching filtered data:', error);
        this.isLoading = false;
      }
    );
  }

  exportToExcel() {
    /* generate worksheet */
    const ws: XLSX.WorkSheet = XLSX.utils.table_to_sheet(this.complianceHoldTable.nativeElement);

    /* generate workbook and add the worksheet */
    const wb: XLSX.WorkBook = XLSX.utils.book_new();
    XLSX.utils.book_append_sheet(wb, ws, 'Sheet1');

    /* save to file */
    const fileName = `Compliance_Hold_Transactions${Date.now()}.xlsx`;
    XLSX.writeFile(wb, fileName);
  }

  transformData() {
    this.allComplianceTransactions = this.allComplianceTransactions.map((item) => {
      const newItem = {};
      for (const key in item) {
        if (item.hasOwnProperty(key)) {
          const newKey = key.replace(/^\|(?<=,)\|\*(?=,|$)|\s+/g, ''); // Remove asterisk and replace spaces with underscores
          newItem[newKey] = item[key];
        }
      }
      return newItem;
    });
  }
}
